@@ -3,8 +3,6 @@ import tempfile
 import subprocess
 import string
 
-import multidict
-
 from cursedspace import ShellContext, InputLine
 
 import metaindex.shared
@@ -12,6 +10,7 @@ import metaindex.indexer
 import metaindex.indexers
 import metaindex.ocr
 import metaindex.stores
+from metaindex import index_files, CacheEntry
 
 from metaindexmanager import layouts
 from metaindexmanager import utils
@@ -332,6 +331,8 @@ class BookmarkNameReader(InputLine):
 
     def handle_key(self, key):
         self.app.activate_panel(self.app.previous_focus)
+        self.win.erase()
+        self.win.noutrefresh()
         self.destroy()
         if len(str(key)) == 1 and str(key) in string.ascii_letters:
             self.bookmark_selected(str(key))
@@ -497,8 +498,6 @@ class SetCommand(Command):
     NAME = 'set'
 
     def completion_options(self, context, *args):
-        logger.debug("Completion base: %s", args)
-
         options = []
         if len(args) < 2:
             text = args[0] if len(args) > 0 else ""
@@ -653,11 +652,10 @@ class RunIndexers(Command):
         else:
             paths = [path]
 
-        results = metaindex.indexer.index_files(paths,
-                                                processes=1,
-                                                ocr_=metaindex.ocr.Dummy(),
-                                                fulltext=False,
-                                                config=application.metaindexconf)
+        results = index_files(paths,
+                              application.metaindexconf,
+                              ocr=metaindex.ocr.Dummy(),
+                              fulltext=False)
         if len(results) == 0:
             return
 
@@ -665,24 +663,24 @@ class RunIndexers(Command):
 
         # merge the results
         for idx, result in enumerate(results):
-            lpath, success, extra = result
+            logger.debug("index resulted in %s", result)
 
-            base = application.cache.get(lpath, False)
+            base = application.cache.get(result.filename)
             if len(base) == 0:
-                info = multidict.MultiDict()
+                info = CacheEntry(result.filename)
             else:
-                info = base[0][1]
+                info = base[0]
 
-            if not success:
-                logger.debug(f"Indexer did not succeed on {lpath}")
+            if not result.success:
+                logger.debug(f"Indexer did not succeed on {result.filename}")
                 continue
-            logger.debug(f"Indexer found something for {lpath}")
+            logger.debug(f"Indexer found something for {result.filename}")
 
             # extend the cached metadata with the newly indexed data
             newly_added = False
-            for key in set(extra.keys()):
-                for value in extra.getall(key):
-                    if value in info.getall(key, []):
+            for key in set(result.info.keys()):
+                for value in result.info[key]:
+                    if value in info[key]:
                         continue
                     info.add(key, value)
                     newly_added = True
@@ -691,13 +689,13 @@ class RunIndexers(Command):
                 logger.debug("Nothing new here")
                 return
 
-            application.cache.insert(lpath, info)
+            application.cache.insert(info)
             blocker.progress((idx+1, len(results)))
 
         if isinstance(panel, (DocPanel,)):
             panel.search(panel.query)
             application.callbacks.put((panel,
-                                       lambda: panel.jump_to(item)))
+                                       lambda: panel.jump_to(item[-1].path)))
         elif isinstance(panel, (EditorPanel,)):
             application.callbacks.put((panel,
                                        panel.reload))
