@@ -8,19 +8,20 @@ from metaindexmanager import command
 from metaindexmanager import utils
 from metaindexmanager import clipboard
 from metaindexmanager.utils import logger
-from metaindexmanager.panel import ListPanel
+from metaindexmanager.panel import ListPanel, register
 
 
 DEFAULT_COLUMNS = "title filename tags+ mimetype"
 
 
+@register
 class DocPanel(ListPanel):
     SCOPE = 'documents'
 
     CONFIG_COLUMNS = 'columns'
 
-    def __init__(self, *args, searchterm='', **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, application, searchterm=''):
+        super().__init__(application)
         self.col_widths = []
         self.fieldkeys = []
         self.query = None
@@ -30,6 +31,8 @@ class DocPanel(ListPanel):
         self.configuration_changed()
         self.search(searchterm)
 
+        self.show_overflow_error = False
+
     @property
     def synonyms(self):
         return self.app.metaindexconf.synonyms
@@ -96,8 +99,9 @@ class DocPanel(ListPanel):
         self.items = [item for item in self.items if sum([len(field) for field in item[:-1]]) > 0]
         self.items.sort(key=lambda i: [len(i[0]) == 0] + [v.lower() for v in i[:-1]])
 
-    def make_row(self, result):
+    def make_row(self, entry):
         fields = []
+        logger.debug("make_row %s", entry.metadata)
         for key in self.fieldkeys:
             multivalue = False
             if key.endswith('+'):
@@ -107,21 +111,22 @@ class DocPanel(ListPanel):
             if key == 'icon':
                 # the '{}' parameter prevents that path.stat() is being called,
                 # speeding up the display by a lot
-                values = [utils.get_ls_icon(pathlib.Path(result[0]), {})]
+                values = [utils.get_ls_icon(pathlib.Path(entry.path), {})]
             else:
                 values = []
                 for expandedkey in set(self.synonyms.get(key, [key])):
-                    values += result[1].getall(expandedkey, [])
+                    values += [self.app.as_printable(v)
+                               for v in entry[expandedkey]]
 
             if len(values) == 0:
                 fields.append("")
             elif len(values) > 1 and multivalue:
-                fields.append(', '.join([self.app.humanize(v) for v in values if len(v) > 0]))
+                fields.append(', '.join([v for v in values if len(v) > 0]))
             else:
-                fields.append(self.app.humanize(values[0]))
+                fields.append(values[0])
             fields[-1] = fields[-1].replace("\n", " ").replace("\r", " ").replace("\t", " ")
 
-        fields.append(result)
+        fields.append(entry)
         return fields
 
     def calculate_grid_size(self):
@@ -151,6 +156,8 @@ class DocPanel(ListPanel):
         self.col_widths[-1] = maxwidth - sum(self.col_widths[:-1])
 
         assert sum(self.col_widths) <= maxwidth
+        logger.debug("Resized columns: %s (maxwidth: %s)",
+                     self.col_widths, maxwidth)
 
     def resize(self, *args):
         super().resize(*args)
@@ -183,7 +190,6 @@ class DocPanel(ListPanel):
         # paint value per column
         for column in range(len(self.fieldkeys)):
             if self.col_widths[column] < 2:
-                logger.debug(" ... no")
                 continue
             text = item[column]
             if column == 0 and item in self.multi_selection:
@@ -191,7 +197,13 @@ class DocPanel(ListPanel):
             text = text[:self.col_widths[column]]
             if column < len(self.fieldkeys) - 1 and len(text) >= self.col_widths[column]:
                 text = text[:-1] + ' '
-            assert len(text) + x <= maxwidth
+            if len(text) + x > maxwidth:
+                text = text[:maxwidth-x]
+                if self.show_overflow_error:
+                    self.show_overflow_error = False
+                    logger.error("text of size %s (offset %s) does not fit "
+                                 "in column %s (%s) with maxwidth %s",
+                                 len(text), x, column, self.col_widths, maxwidth)
             try:
                 self.win.addstr(y, x, text, attr)
             except curses.error:
@@ -206,6 +218,7 @@ class DocPanel(ListPanel):
             self.search(path)
 
         if self.is_busy:
+            logger.debug("postponing jump to %s", item)
             self.post_load = lambda: self.do_jump_to(item)
             return
 
@@ -214,8 +227,8 @@ class DocPanel(ListPanel):
     def do_jump_to(self, item):
         targetitem = str(item)
 
-        for cursor, item in enumerate(self.items):
-            if item[-1][0] == targetitem:
+        for cursor, rowitem in enumerate(self.items):
+            if str(rowitem[-1].path) == targetitem:
                 self.cursor = cursor
                 break
 
@@ -227,18 +240,18 @@ class DocPanel(ListPanel):
     def selected_path(self):
         result = self.selected_item
         if result is not None:
-            result = pathlib.Path(result[-1][0])
+            result = pathlib.Path(result[-1].path)
         return result
 
     @property
     def selected_paths(self):
-        return [pathlib.Path(i[-1][0]) for i in self.selected_items]
+        return [pathlib.Path(i[-1].path) for i in self.selected_items]
 
     def open_selected(self):
         if self.selected_item is None:
             self.app.error("Nothing selected")
             return
-        path = pathlib.Path(self.selected_item[-1][0])
+        path = pathlib.Path(self.selected_item[-1].path)
         if path.is_file():
             self.app.open_file(path)
         else:
