@@ -16,7 +16,7 @@ from metaindexmanager import command
 from metaindexmanager import utils
 from metaindexmanager import clipboard
 from metaindexmanager.utils import logger
-from metaindexmanager.panel import ListPanel
+from metaindexmanager.panel import ListPanel, register
 
 
 class Line:
@@ -97,13 +97,15 @@ class FolderObserver(threading.Thread):
         self.join()
 
 
+@register
 class FilePanel(ListPanel):
     SCOPE = 'files'
 
     CONFIG_ICONS = 'use-icons'
 
-    def __init__(self, *args, path='.', **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, application, path='.'):
+        super().__init__(application)
+        path = pathlib.Path(path)
         self.path = None
         self.previous_path = None
         self.post_load = None
