@@ -33,7 +33,7 @@ from . import utils
 from . import version
 from . import shared
 from .utils import logger
-from .panel import LIST_SCROLL_COMMANDS
+from .panel import LIST_SCROLL_COMMANDS, find_panel_type
 from .command import resolve_command, Command
 from .docpanel import DocPanel
 from .filepanel import FilePanel
@@ -207,7 +207,10 @@ class Application(cursedspace.Application):
         self.named_clipboard = {}
         # bookmarks are tuples (panel type, base path, selected item)
         self.bookmarks = {}
-        self.restore_bookmarks()
+        try:
+            self.restore_bookmarks()
+        except Exception as exc:
+            logger.debug("Failed to restore bookmarks: %s", exc)
 
         # during command execution paints are not executed, but enqueued
         self.prevent_paint = False
@@ -258,7 +261,14 @@ class Application(cursedspace.Application):
                     self.panels.append(DocPanel(self, searchterm=location))
             self.current_panel = self.panels[-1]
         else:
-            self.current_panel = DocPanel(self)
+            default_view = self.configuration.get(shared.ALL_SCOPE,
+                                                  shared.PREF_DEFAULT_PANEL,
+                                                  shared.DEFAULT_DEFAULT_PANEL)
+            logger.debug("default view configured to %s", default_view)
+            paneltype = find_panel_type(default_view)
+            if paneltype is None:
+                paneltype = DocPanel
+            self.current_panel = paneltype(self)
             self.panels.append(self.current_panel)
         self.resize_panels()
         self.paint()
@@ -667,6 +677,7 @@ class Application(cursedspace.Application):
         """Store this bookmark"""
         self.bookmarks[mark] = (type(panel), path, item)
         self.store_bookmarks()
+        self.clear_info_text()
 
     def load_bookmark(self, mark):
         """Restore this bookmark, create a panel if required"""
@@ -681,12 +692,10 @@ class Application(cursedspace.Application):
            (not hasattr(self.current_panel, 'is_busy') or not self.current_panel.is_busy):
             self.current_panel.jump_to(item, path)
         else:
-            if issubclass(paneltype, FilePanel):
-                panel = paneltype(self, path=path)
-            elif issubclass(paneltype, DocPanel):
-                panel = paneltype(self, searchterm=path)
-            else:
-                msg = f"Unexpected panel type in bookmarks: {paneltype}"
+            try:
+                panel = paneltype(self, path)
+            except Exception as exc:
+                msg = f"Error loading bookmark for {paneltype}: {exc}"
                 logger.error(msg)
                 self.error(msg)
                 return
@@ -694,13 +703,14 @@ class Application(cursedspace.Application):
             self.activate_panel(panel)
 
             panel.jump_to(item)
+        self.clear_info_text()
 
     def restore_bookmarks(self):
         """Load the previously defined bookmarks from disk
 
         Bookmarks are stored one per line in plain text in this format:
 
-            <mark> [f|d] <path> <path to item>
+            <mark> <panel type> <path> <path to item>
 
         """
         if not BOOKMARKFILE.is_file():
@@ -710,8 +720,6 @@ class Application(cursedspace.Application):
             parts = shlex.split(line)
             if len(parts) != 4:
                 continue
-            if parts[1] not in 'fd':
-                continue
 
             mark, typename, path, item = parts
             if len(item) == 0:
@@ -719,14 +727,10 @@ class Application(cursedspace.Application):
             if item is not None:
                 item = pathlib.Path(item)
 
-            paneltype = None
-            if typename == 'f':
-                paneltype = FilePanel
-                path = pathlib.Path(path)
-            elif typename == 'd':
-                paneltype = DocPanel
-            else:
-                raise NotImplementedError(f"Unknown panel type {typename}")
+            paneltype = find_panel_type(typename)
+            if paneltype is None:
+                logger.warning(f"Unknown panel type {typename}")
+                continue
 
             self.bookmarks[mark] = (paneltype, path, item)
 
@@ -734,7 +738,7 @@ class Application(cursedspace.Application):
         """Save the bookmarks to disk"""
         with open(BOOKMARKFILE, 'wt', encoding='utf-8') as fh:
             for mark in sorted(self.bookmarks.keys()):
-                typename = 'd' if self.bookmarks[mark][0] is DocPanel else 'f'
+                typename = self.bookmarks[mark][0].SCOPE
                 line = [mark, typename] + [str(v) for v in self.bookmarks[mark][1:]]
                 fh.write(shlex.join(line) + "\n")
 
@@ -759,7 +763,6 @@ class Application(cursedspace.Application):
                 self.layout.resize_panels()
                 must_repaint = True
             elif scope == shared.ALL_SCOPE and name == PREF_INFO_TIMEOUT:
-                logger.debug("GNA!")
                 raw_value = self.configuration.get(scope, name, DEFAULT_INFO_TIMEOUT)
                 try:
                     value = utils.parse_duration(raw_value)
@@ -898,19 +901,21 @@ class Application(cursedspace.Application):
         self.current_panel.win.noutrefresh()
         curses.doupdate()
 
-    def humanize(self, value):
-        """Convert any metadata value to a human-friendly string"""
-        if isinstance(value, datetime.datetime):
-            value = value.strftime("%Y-%m-%d %H:%M:%S")
-        elif isinstance(value, datetime.date):
-            value = value.strftime("%Y-%m-%d")
-        elif isinstance(value, (int, float, bool)):
-            value = str(value)
-        elif not isinstance(value, str):
-            logger.debug("Unexpected type '%s' to humanize", type(value))
-            value = str(value)
+    @staticmethod
+    def as_printable(value, shorten=100):
+        """Clean up a string to make it printable
 
-        return (''.join(letter for letter in value if ord(letter) > 31)).strip()
+        Shorten the value to ``shorten`` characters.
+        If ``shorten`` is None, no shortening will happend.
+        """
+        if value is None:
+            return ''
+        text = str(value)
+        if shorten is not None:
+            text = text[:shorten]
+        return (''.join(letter
+                        for letter in text
+                        if letter.isprintable())).strip().replace('\t', ' ')
 
     @property
     def collection_metadata(self):
