@@ -26,6 +26,31 @@ LIST_SCROLL_COMMANDS = [
 OPTION_PAGING = 'paging'
 OPTION_SCROLL_MARGIN = 'scroll-margin'
 
+_registered_panels = {}
+
+
+def register(cls):
+    """Decorator to register a panel to metaindexmanager
+
+    The panel will be identified by its ``SCOPE`` class property.
+
+    The purpose of this registration is to be used in bookmarks
+    and the ``all.default-view`` settings.
+    """
+    if hasattr(cls, 'SCOPE'):
+        assert cls.SCOPE not in _registered_panels
+        _registered_panels[cls.SCOPE] = cls
+    return cls
+
+
+def find_panel_type(name):
+    """Find a registered panel type by its name
+
+    :param name: ``SCOPE`` name of the requested panel type
+    :return: The type of the registered panel, or ``None``
+    """
+    return _registered_panels.get(name, None)
+
 
 class BlockingTask(ScrollPanel):
     INDICATOR_1 = '-\\|/'
@@ -249,6 +274,12 @@ class ListPanel(ScrollPanel):
 
 
 def extract_key_sequence(application, commandname):
+    """Get all key sequences of this command in the 'any' scope
+
+    :param application: The application
+    :param commandname: The name of the command in question
+    :rtype: ``list[tuple[str]]``, all key identifiers
+    """
     return [keys
             for scope, keys, cmd in application.keys
-            if cmd[0] == commandname and scope == 'any']
+            if cmd[0] == commandname and scope == shared.ANY_SCOPE]
