@@ -9,3 +9,6 @@ PREF_BORDER = 'border'
 DEFAULT_BORDER = 'full'
 BORDER_FULL = 'full'
 BORDER_MINIMAL = 'minimal'
+
+PREF_DEFAULT_PANEL = 'default-panel'
+DEFAULT_DEFAULT_PANEL = 'documents'
